// Copyright 2022 Bitcoding

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.nativeAds.MaxNativeAdView;
import com.applovin.mediation.nativeAds.MaxNativeAdViewBinder;
import com.google.android.gms.ads.nativead.MediaView;
import com.google.android.gms.ads.nativead.NativeAd;
import com.google.android.gms.ads.nativead.NativeAdView;
import com.btcpiyush.ads.google_applovin_unity_ads.GoogleApplovinUnityAdsPlugin.NativeAdFactory;

import java.util.Map;

public class CustomNativeAd implements NativeAdFactory {
  private final LayoutInflater layoutInflater;

 public CustomNativeAd(LayoutInflater layoutInflater) {
    this.layoutInflater = layoutInflater;
  }

  @Override
  public NativeAdView createNativeAd(NativeAd nativeAd, Map<String, Object> customOptions) {
    final NativeAdView adView = (NativeAdView) layoutInflater.inflate(R.layout.native_full, null);

    // Set the media view.
    adView.setMediaView((MediaView) adView.findViewById(R.id.media_view1));

    // Set other ad assets.
    adView.setHeadlineView(adView.findViewById(R.id.primary));
    adView.setBodyView(adView.findViewById(R.id.body_text));
    adView.setCallToActionView(adView.findViewById(R.id.cta));
    adView.setIconView(adView.findViewById(R.id.icon));
//    adView.setPriceView(adView.findViewById(R.id.ad_price));
//    adView.setStarRatingView(adView.findViewById(R.id.ad_stars));
    adView.setStoreView(adView.findViewById(R.id.body));
//    adView.setAdvertiserView(adView.findViewById(R.id.ad_advertiser));

    // The headline and mediaContent are guaranteed to be in every NativeAd.
    ((TextView) adView.getHeadlineView()).setText(nativeAd.getHeadline());
    if (nativeAd.getMediaContent() == null) {
      adView.getMediaView().setVisibility(View.GONE);
    } else {
      adView.getMediaView().setVisibility(View.VISIBLE);
      adView.getMediaView().setMediaContent(nativeAd.getMediaContent());
    }

    // These assets aren't guaranteed to be in every NativeAd, so it's important to
    // check before trying to display them.
    if (nativeAd.getBody() == null) {
      adView.getBodyView().setVisibility(View.INVISIBLE);
    } else {
      adView.getBodyView().setVisibility(View.VISIBLE);
      ((TextView) adView.getBodyView()).setText(nativeAd.getBody());
    }

    if (nativeAd.getCallToAction() == null) {
      adView.getCallToActionView().setVisibility(View.INVISIBLE);
    } else {
      adView.getCallToActionView().setVisibility(View.VISIBLE);
      ((Button) adView.getCallToActionView()).setText(nativeAd.getCallToAction());
    }

    if (nativeAd.getIcon() == null) {
      adView.getIconView().setVisibility(View.GONE);
    } else {
      ((ImageView) adView.getIconView()).setImageDrawable(nativeAd.getIcon().getDrawable());
      adView.getIconView().setVisibility(View.VISIBLE);
    }

   /* if (nativeAd.getPrice() == null) {
      adView.getPriceView().setVisibility(View.INVISIBLE);
    } else {
      adView.getPriceView().setVisibility(View.VISIBLE);
      ((TextView) adView.getPriceView()).setText(nativeAd.getPrice());
    }*/

    if (nativeAd.getStore() == null) {
      adView.getStoreView().setVisibility(View.INVISIBLE);
    } else {
      adView.getStoreView().setVisibility(View.VISIBLE);
      ((TextView) adView.getStoreView()).setText(nativeAd.getStore());
    }

    /*if (nativeAd.getStarRating() == null) {
      adView.getStarRatingView().setVisibility(View.INVISIBLE);
    } else {
      ((RatingBar) adView.getStarRatingView()).setRating(nativeAd.getStarRating().floatValue());
      adView.getStarRatingView().setVisibility(View.VISIBLE);
    }*/

   /* if (nativeAd.getAdvertiser() == null) {
      adView.getAdvertiserView().setVisibility(View.INVISIBLE);
    } else {
      adView.getAdvertiserView().setVisibility(View.VISIBLE);
      ((TextView) adView.getAdvertiserView()).setText(nativeAd.getAdvertiser());
    }*/

    // This method tells the Google Mobile Ads SDK that you have finished populating your
    // native ad view with this native ad.
    adView.setNativeAd(nativeAd);

    return adView;
  }

  @Override
  public MaxNativeAdView createAppLovinNativeAd(@NonNull Activity context) {
    MaxNativeAdViewBinder binder =
            new MaxNativeAdViewBinder.Builder(R.layout.native_full_template)
                    .setTitleTextViewId(R.id.primary)
                    .setBodyTextViewId(R.id.body_text)
                    .setAdvertiserTextViewId(R.id.body)
                    .setIconImageViewId(R.id.icon)
                    .setMediaContentViewGroupId(R.id.media_view)
                    .setCallToActionButtonId(R.id.cta)
                    .build();
    return new MaxNativeAdView(binder, context);
  }


}
