// Copyright 2022 Bitcoding
// See the License for the specific language governing permissions and
// limitations under the License.

package com.btcpiyush.ads.google_applovin_unity_ads;

import android.app.Activity;
import android.content.Context;

import androidx.annotation.NonNull;

import com.applovin.mediation.MaxAdListener;
import com.applovin.mediation.MaxRewardedAdListener;
import com.applovin.mediation.ads.MaxInterstitialAd;
import com.applovin.mediation.ads.MaxRewardedAd;
import com.applovin.mediation.nativeAds.MaxNativeAdListener;
import com.applovin.mediation.nativeAds.MaxNativeAdLoader;
import com.applovin.mediation.nativeAds.MaxNativeAdView;
import com.applovin.sdk.AppLovinSdk;
import com.google.android.gms.ads.AdListener;
import com.google.android.gms.ads.AdLoader;
import com.google.android.gms.ads.AdRequest;
import com.google.android.gms.ads.admanager.AdManagerAdRequest;
import com.google.android.gms.ads.admanager.AdManagerInterstitialAd;
import com.google.android.gms.ads.admanager.AdManagerInterstitialAdLoadCallback;
import com.google.android.gms.ads.appopen.AppOpenAd;
import com.google.android.gms.ads.appopen.AppOpenAd.AppOpenAdLoadCallback;
import com.google.android.gms.ads.interstitial.InterstitialAd;
import com.google.android.gms.ads.interstitial.InterstitialAdLoadCallback;
import com.google.android.gms.ads.nativead.NativeAd.OnNativeAdLoadedListener;
import com.google.android.gms.ads.nativead.NativeAdOptions;
import com.google.android.gms.ads.rewarded.RewardedAd;
import com.google.android.gms.ads.rewarded.RewardedAdLoadCallback;
import com.google.android.gms.ads.rewardedinterstitial.RewardedInterstitialAd;
import com.google.android.gms.ads.rewardedinterstitial.RewardedInterstitialAdLoadCallback;

/**
 * A wrapper around load methods in GMA. This exists mainly to make the Android code more testable.
 */
class FlutterAdLoader {

  @NonNull private final Activity context;

  public FlutterAdLoader(@NonNull Activity context) {
    this.context = context;
  }

  /** Load an app open ad. */
  public void loadAppOpen(
      @NonNull String adUnitId,
      @NonNull AdRequest adRequest,
      int orientation,
      @NonNull AppOpenAdLoadCallback loadCallback) {
    AppOpenAd.load(context, adUnitId, adRequest, orientation, loadCallback);
  }

  /** Load an ad manager app open ad. */
  public void loadAdManagerAppOpen(
      @NonNull String adUnitId,
      @NonNull AdManagerAdRequest adRequest,
      int orientation,
      @NonNull AppOpenAdLoadCallback loadCallback) {
    AppOpenAd.load(context, adUnitId, adRequest, orientation, loadCallback);
  }

  /** Load an interstitial ad. */
  public void loadInterstitial(
      @NonNull String adUnitId,
      @NonNull AdRequest adRequest,
      @NonNull InterstitialAdLoadCallback loadCallback) {
    InterstitialAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load an ad manager interstitial ad. */
  public void loadAdManagerInterstitial(
      @NonNull String adUnitId,
      @NonNull AdManagerAdRequest adRequest,
      @NonNull AdManagerInterstitialAdLoadCallback loadCallback) {
    AdManagerInterstitialAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load an AppLovin interstitial ad. */
  public MaxInterstitialAd loadAppLovinInterstitial(
          @NonNull Activity act,
          @NonNull String adUnitId,
          @NonNull AppLovinSdk _appLovinSDK,
          @NonNull MaxAdListener loadCallback) {
    Activity activity = context == null ? act : context;
    MaxInterstitialAd appLovinInterstitialAd = new MaxInterstitialAd(
            adUnitId, _appLovinSDK, activity
    );
    appLovinInterstitialAd.setListener(loadCallback);
    appLovinInterstitialAd.loadAd();
    return  appLovinInterstitialAd;
  }

  /** Load a rewarded ad. */
  public void loadRewarded(
      @NonNull String adUnitId,
      @NonNull AdRequest adRequest,
      @NonNull RewardedAdLoadCallback loadCallback) {
    RewardedAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load a rewarded ad. */
  public MaxRewardedAd loadAppLovinRewarded(
      @NonNull Activity act,
      @NonNull String adUnitId,
      @NonNull AppLovinSdk _appLovinSDK,
      @NonNull MaxRewardedAdListener loadCallback) {
    Activity activity = context == null ? act : context;
    MaxRewardedAd appLovinRewardAd = MaxRewardedAd.getInstance(adUnitId, _appLovinSDK, activity);
    appLovinRewardAd.setListener(loadCallback);
    appLovinRewardAd.loadAd();
    return appLovinRewardAd;
  }

  /** Load a rewarded interstitial ad. */
  public void loadRewardedInterstitial(
      @NonNull String adUnitId,
      @NonNull AdRequest adRequest,
      @NonNull RewardedInterstitialAdLoadCallback loadCallback) {
    RewardedInterstitialAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load an ad manager rewarded ad. */
  public void loadAdManagerRewarded(
      @NonNull String adUnitId,
      @NonNull AdManagerAdRequest adRequest,
      @NonNull RewardedAdLoadCallback loadCallback) {
    RewardedAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load an ad manager rewarded interstitial ad. */
  public void loadAdManagerRewardedInterstitial(
      @NonNull String adUnitId,
      @NonNull AdManagerAdRequest adRequest,
      @NonNull RewardedInterstitialAdLoadCallback loadCallback) {
    RewardedInterstitialAd.load(context, adUnitId, adRequest, loadCallback);
  }

  /** Load a native ad. */
  public void loadNativeAd(
      @NonNull String adUnitId,
      @NonNull OnNativeAdLoadedListener onNativeAdLoadedListener,
      @NonNull NativeAdOptions nativeAdOptions,
      @NonNull AdListener adListener,
      @NonNull AdRequest adRequest) {
    new AdLoader.Builder(context, adUnitId)
        .forNativeAd(onNativeAdLoadedListener)
        .withNativeAdOptions(nativeAdOptions)
        .withAdListener(adListener)
        .build()
        .loadAd(adRequest);
  }

  /** Load a native ad. */
  public MaxNativeAdLoader loadAppLovinNativeAd(
      @NonNull Activity act,
      @NonNull String adUnitId,
      @NonNull AppLovinSdk _appLovinSDK,
      @NonNull MaxNativeAdListener onNativeAdLoadedListener,
      @NonNull MaxNativeAdView onNativeAdView
      ) {

    Activity activity = context == null ? act : context;

    MaxNativeAdLoader appLovinNativeAdView  = new MaxNativeAdLoader(adUnitId,  _appLovinSDK, activity);
    appLovinNativeAdView.setNativeAdListener(onNativeAdLoadedListener);
    appLovinNativeAdView.loadAd(onNativeAdView);

    return appLovinNativeAdView;
  }

  /** Load an ad manager native ad. */
  public void loadAdManagerNativeAd(
      @NonNull String adUnitId,
      @NonNull OnNativeAdLoadedListener onNativeAdLoadedListener,
      @NonNull NativeAdOptions nativeAdOptions,
      @NonNull AdListener adListener,
      @NonNull AdManagerAdRequest adManagerAdRequest) {
    new AdLoader.Builder(context, adUnitId)
        .forNativeAd(onNativeAdLoadedListener)
        .withNativeAdOptions(nativeAdOptions)
        .withAdListener(adListener)
        .build()
        .loadAd(adManagerAdRequest);
  }
}
