import 'package:get/instance_manager.dart';
import 'package:get/route_manager.dart';
import 'package:google_applovin_unity_ads/ads/SettingsModel.dart';
import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';
import 'package:flutter/widgets.dart';

const AdRequest _request = AdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

const AdManagerAdRequest _ad_manager_request = AdManagerAdRequest(
    // keywords: <String>['foo', 'bar'],
    // contentUrl: 'http://foo.com/bar.html',
    // nonPersonalizedAds: true,
    );

BannerAd? _bannerAd;
NativeAd? _nativeAd;
AppLovinNativeAd? _appLovinnativeAd;
AdManagerBannerAd? _adManagerBannerAd;
AppLovinBannerAd? _appLovinBannerAd;

const String _GOOGLEAD = "google_ads";
const String _ADMANAGERAD = "ad_manager";
const String _APPLOVINAD = "app_lovin";
const String _FACEBOOKAD = "facebook_ads";
const String _CUSTOMAD = "custom_ads";

Settings? _settings = getAdsSettings();
int appVersionCode = getAppVersionCode();

void destroyAd() {
  getInterestitialAd()?.dispose();
  getAdManagerInterstitialAd()?.dispose();
  getRewardedAd()?.dispose();
  _adManagerBannerAd?.dispose();
  _bannerAd?.dispose();
  _appLovinBannerAd?.dispose();
  _appLovinnativeAd?.dispose();
  _nativeAd?.dispose();
}

initOpenAds({Function? onOpenAdLoaded, Function? onOpenAdFailedLoaded}) {
  if (_settings == null) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onOpenAdLoaded != null) {
      onOpenAdLoaded();
    }
    return;
  }


  AppOpenAdManager appOpenAdManager = AppOpenAdManager()
    ..loadAd(callback: onOpenAdLoaded);
  setAppLifecycleReactor(AppLifecycleReactor(
      appOpenAdManager: appOpenAdManager, appVersionCode: appVersionCode));
  getAppLifecycleReactor()?.listenToAppStateChanges();
}

showOpenAds() {
  if (_settings == null) {
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    return;
  }

  getAppLifecycleReactor()?.appOpenAdManager.showAdIfAvailable();
}

showIntraAds({Function? callback}) async {
  showProgressbarDialog(Get.context!);
  _settings?.adSetting?.interestialAdsIntervalCount =
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) + 1;
  loadIntraAds(
      onAdLoadedCallback: () => {Get.back(), _showInterstitialAd(callback)});
}

showRewardAds({Function? callback, Function? onFailedCallback}) async {
  showProgressbarDialog(Get.context!);
  loadRewardAds(
      onAdLoadedCallback: () => {Get.back(), _showRewardedAd(callback)},
      onAdFailedCallback: () => {
            Get.back(),
            if (onFailedCallback != null) {onFailedCallback()}
          });
}

showBannerAds({Ad? banner, int pos = 0, AdSize size = AdSize.banner}) {
  if (banner != null) {
    if(banner is AdWithView) {
      return Container(
        width: size.width.toDouble(),
        height: size.height.toDouble(),
        child: AdWidget(ad: banner),
      );
    }
  }
  return Container(height: 0);
}

showBannerAdsGrid({int pos = 0, AdSize size = AdSize.banner}) {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    return Container(
      height: 0,
    );
  }

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        _bannerAd = BannerAd(
            size: size,
            adUnitId: _settings?.googleAds?.bannerId ?? "",
            listener: BannerAdListener(
              onAdLoaded: (Ad ad) {
                print('$BannerAd loaded.');
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$BannerAd failedToLoad: $error');
                ad.dispose();
                _bannerAd = null;
                showBannerAdsGrid(pos: pos + 1);
              },
              onAdOpened: (Ad ad) => print('$BannerAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$BannerAd onAdClosed.'),
            ),
            request: _request)..load();

        return Container(
          width: size.width.toDouble(),
          height: size.height.toDouble(),
          child: AdWidget(ad: _bannerAd!),
        );
      } else {
        showBannerAdsGrid(pos: pos + 1);
      }
      break;
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        _adManagerBannerAd = AdManagerBannerAd(
          adUnitId: _settings?.googleAds?.bannerId ?? "",
          sizes: <AdSize>[size],
          request: _ad_manager_request,
          listener: AdManagerBannerAdListener(
            onAdLoaded: (Ad ad) async {
              print('Inline adaptive banner loaded: ${ad.responseInfo}');

              /* AdManagerBannerAd bannerAd = (ad as AdManagerBannerAd);
              final AdSize? size = await bannerAd.getPlatformAdSize();

              if (size == null) {
                print('Error: getPlatformAdSize() returned null for $bannerAd');
                return;
              }*/
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              _adManagerBannerAd = null;
              showBannerAdsGrid(pos: pos + 1);
            },
          ),
        )..load();
        return Container(
          width: size.width.toDouble(),
          height: size.height.toDouble(),
          child: AdWidget(ad: _adManagerBannerAd!),
        );
      } else {
        showBannerAdsGrid(pos: pos + 1);
      }
      break;
    case _APPLOVINAD:
      if (_settings?.appLovin?.isBannerAds ?? false) {
        _appLovinBannerAd = AppLovinBannerAd(
          adUnitId: _settings?.appLovin?.bannerId ?? "",
          size: size,
          listener: AdManagerBannerAdListener(
            onAdLoaded: (Ad ad) async {
              print('Inline adaptive banner loaded: ${ad.responseInfo}');
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              _appLovinBannerAd = null;
              showBannerAdsGrid(pos: pos + 1);
            },
          ),
        )..load();

        return Container(
          width: size.width.toDouble(),
          height: size.height.toDouble(),
          child: AdWidget(ad: _appLovinBannerAd!),
        );
      } else {
        showBannerAdsGrid(pos: pos + 1);
      }
      break;
    default:
      showBannerAdsGrid(pos: pos + 1);
      break;
  }
}

loadBannerAds(
    {int pos = 0,
    AdSize size = AdSize.banner,
    Function(Ad)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        BannerAd(
                size: size,
                adUnitId: _settings?.googleAds?.bannerId ?? "",
                listener: BannerAdListener(
                  onAdLoaded: (Ad ad) {
                    print('$ad loaded.');
                    if (onAdLoadedCallback != null) {
                      onAdLoadedCallback(ad);
                    }
                  },
                  onAdFailedToLoad: (Ad ad, LoadAdError error) {
                    print('$BannerAd failedToLoad: $error');
                    ad.dispose();
                    loadBannerAds(
                        pos: pos + 1,
                        size: size,
                        onAdLoadedCallback: onAdLoadedCallback,
                        onAdFailedToLoadCallback: onAdFailedToLoadCallback);
                  },
                  onAdOpened: (Ad ad) => print('$BannerAd onAdOpened.'),
                  onAdClosed: (Ad ad) => print('$BannerAd onAdClosed.'),
                ),
                request: _request)
            .load();
      } else {
        loadBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isBannerAds ?? false) {
        AdManagerBannerAd(
          adUnitId: _settings?.googleAds?.bannerId ?? "",
          sizes: <AdSize>[size],
          request: _ad_manager_request,
          listener: AdManagerBannerAdListener(
            onAdLoaded: (Ad ad) async {
              print('Inline adaptive banner loaded: ${ad.responseInfo}');
              if (onAdLoadedCallback != null) {
                onAdLoadedCallback(ad);
              }
              // After the ad is loaded, get the platform ad size and use it to
              // update the height of the container. This is necessary because the
              // height can change after the ad is loaded.

              /* AdManagerBannerAd bannerAd = (ad as AdManagerBannerAd);
              final AdSize? size = await bannerAd.getPlatformAdSize();

              if (size == null) {
                print('Error: getPlatformAdSize() returned null for $bannerAd');
                return;
              }*/
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              // _adManagerBannerAd = null;
              loadBannerAds(
                  pos: pos + 1,
                  size: size,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
          ),
        ).load();
      } else {
        loadBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _APPLOVINAD:
      if (_settings?.appLovin?.isBannerAds ?? false) {
        AppLovinBannerAd(
          adUnitId: _settings?.appLovin?.bannerId ?? "",
          size: size,
          listener: BannerAdListener(
            onAdLoaded: (Ad ad) async {
              print('Inline adaptive banner loaded: ${ad.responseInfo}');
              if (onAdLoadedCallback != null) {
                onAdLoadedCallback(ad);
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print('Inline adaptive banner failedToLoad: $error');
              ad.dispose();
              loadBannerAds(
                  pos: pos + 1,
                  size: size,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
          ),
        ).load();
      } else {
        loadBannerAds(
            pos: pos + 1,
            size: size,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    default:
      loadBannerAds(
          pos: pos + 1,
          size: size,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      break;
  }
}

loadNativeAds(
    {int pos = 0,
    Function(dynamic)? onAdLoadedCallback,
    Function? onAdFailedToLoadCallback}) async {
  if (_settings == null ||
      (_settings?.adSetting?.appVersionCode == appVersionCode) ||
      (_settings?.adsSequence?.isEmpty ?? true) ||
      ((_settings?.adsSequence?.length ?? 0) <= pos)) {
    if (onAdFailedToLoadCallback != null) {
      onAdFailedToLoadCallback();
    }
    return;
  }

  switch (_settings!.adsSequence![pos]) {
    case _GOOGLEAD:
      if (_settings?.googleAds?.isNativeAds ?? false) {
        _nativeAd = NativeAd(
          adUnitId: _settings?.googleAds?.nativeId ?? "",
          request: _request,
          factoryId: 'nativeFactoryId',
          listener: NativeAdListener(
            onAdLoaded: (Ad ad) {
              print('$NativeAd loaded.');
              if (onAdLoadedCallback != null) {
                onAdLoadedCallback(ad);
              }
            },
            onAdFailedToLoad: (Ad ad, LoadAdError error) {
              print(
                  '$NativeAd == = = ==== ==== ==== ==== ==== ==== === failedToLoad: $error');
              ad.dispose();
              _nativeAd = null;
              loadNativeAds(
                  pos: pos + 1,
                  onAdLoadedCallback: onAdLoadedCallback,
                  onAdFailedToLoadCallback: onAdFailedToLoadCallback);
            },
            onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
            onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
          ),
        )..load();
      } else {
        loadNativeAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isNativeAds ?? false) {
        _nativeAd = NativeAd.fromAdManagerRequest(
            adUnitId: _settings?.googleAds?.nativeId ?? "",
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad ad) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null) {
                  onAdLoadedCallback(ad);
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                _nativeAd = null;
                loadNativeAds(
                    pos: pos + 1,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ),
            adManagerRequest: _ad_manager_request)
          ..load();
      } else {
        loadNativeAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    case _APPLOVINAD:
      if (_settings?.appLovin?.isNativeAds ?? false) {
        _appLovinnativeAd = AppLovinNativeAd(
            adUnitId: _settings?.appLovin?.nativeAds?.manual ?? "",
            factoryId: 'nativeFactoryId',
            listener: NativeAdListener(
              onAdLoaded: (Ad ad) {
                print('$NativeAd loaded.');
                if (onAdLoadedCallback != null) {
                  onAdLoadedCallback(ad);
                }
              },
              onAdFailedToLoad: (Ad ad, LoadAdError error) {
                print('$NativeAd failedToLoad: $error');
                ad.dispose();
                _appLovinnativeAd = null;
                loadNativeAds(
                    pos: pos + 1,
                    onAdLoadedCallback: onAdLoadedCallback,
                    onAdFailedToLoadCallback: onAdFailedToLoadCallback);
              },
              onAdOpened: (Ad ad) => print('$NativeAd onAdOpened.'),
              onAdClosed: (Ad ad) => print('$NativeAd onAdClosed.'),
            ))
          ..load();
      } else {
        loadNativeAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      }
      break;
    default:
      loadNativeAds(
          pos: pos + 1,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedToLoadCallback: onAdFailedToLoadCallback);
      break;
  }
}

showNativeAds({required Ad nativeAd, Size size = const Size(double.infinity, 350)}) {
  if (nativeAd != null) {
    if (nativeAd is AdWithView) {
      return SizedBox(
        width: size.width.toDouble(),
        height: size.height.toDouble(),
        child: AdWidget(ad: nativeAd),
      );
    }
  }
  return Container(height: 0);
}

IsFullIntraAds({Function? callback}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    showIntraAds(callback: callback);
    return;
  }
  if (callback != null) {
    callback();
  }
}

showFullNativeAds({required Ad nativeAd,Size? size}) {
  if (_settings?.adSetting?.isFullAds ?? false) {
    return (size != null) ? showNativeAds(nativeAd: nativeAd, size: size) : showNativeAds(nativeAd: nativeAd);
  }
  return Container(height: 0);
}

void loadIntraAds({int pos = 0, Function? onAdLoadedCallback}) {
  if (_settings == null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  final isInterestial = ((_settings?.adSetting?.interestialAdsInterval ?? 0) >
          0 &&
      (_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
              (_settings?.adSetting?.interestialAdsInterval ?? 0) ==
          0 &&
      ((getInterestialAdTimeInterval() +
                  ((_settings?.adSetting?.interestialAdsTimingInterval ?? 0) *
                      1000)) <=
              DateTime.now().millisecondsSinceEpoch ||
          getInterestialAdTimeInterval() == 0));

  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if (((_settings?.googleAds?.interstitialInterval ?? 0) <= 0 &&
              isInterestial) ||
          ((_settings?.googleAds?.interstitialInterval ?? 0) > 0 &&
              ((_settings?.adSetting?.interestialAdsIntervalCount ?? 0) %
                      (_settings?.googleAds?.interstitialInterval ?? 0) ==
                  0))) {
        if (_settings?.googleAds?.isInterstitialAds ?? true) {
          if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
            _createAdManagerInterstitialAd(pos, onAdLoadedCallback);
            return;
          }
          _createInterstitialAd(pos, onAdLoadedCallback);
          return;
        } else {
          loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
        }
      }
      break;
    case _APPLOVINAD:
      if (isInterestial || (_settings?.appLovin?.isUnlimitedAds ?? false)) {
        if (_settings?.appLovin?.isInterstitialAds ?? true) {
          _createAppLovinInterstitialAd(pos, onAdLoadedCallback);
          return;
        }
      } else {
        loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
      }
      break;
    default:
      loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
      break;
  }
}

void loadRewardAds(
    {int pos = 0, Function? onAdLoadedCallback, Function? onAdFailedCallback}) {
  if (_settings == null) {
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.adsSequence?.isEmpty ?? true) {
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if ((_settings?.adsSequence?.length ?? 0) <= pos) {
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }

  if (_settings?.adSetting?.appVersionCode == appVersionCode) {
    if (onAdFailedCallback != null) {
      onAdFailedCallback();
    }
    return;
  }
  print('RewardedAd pos  $pos ');
  switch (_settings?.adsSequence?[pos]) {
    case _GOOGLEAD:
    case _ADMANAGERAD:
      if (_settings?.googleAds?.isRewaredAds ?? false) {
        if (_ADMANAGERAD == _settings?.adsSequence?[pos]) {
          _createAdmanagerRewardedAd(
              pos, onAdLoadedCallback, onAdFailedCallback);
          return;
        }
        _createRewardedAd(pos, onAdLoadedCallback, onAdFailedCallback);
      } else {
        loadRewardAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    case _APPLOVINAD:
      if (_settings?.appLovin?.isRewaredAds ?? false) {
        _createAppLovinRewardedAd(pos, onAdLoadedCallback, onAdFailedCallback);
        return;
      } else {
        loadRewardAds(
            pos: pos + 1,
            onAdLoadedCallback: onAdLoadedCallback,
            onAdFailedCallback: onAdFailedCallback);
      }
      break;
    default:
      loadRewardAds(
          pos: pos + 1,
          onAdLoadedCallback: onAdLoadedCallback,
          onAdFailedCallback: onAdFailedCallback);
      break;
  }
}

void _createInterstitialAd(int pos, Function? onAdLoadedCallback) {
  if (getInterestitialAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }

    return;
  }
  InterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _request,
      adLoadCallback: InterstitialAdLoadCallback(
        onAdLoaded: (InterstitialAd ad) {
          print('$ad loaded');
          setInterestitialAd(ad);
          getInterestitialAd()?.setImmersiveMode(true);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setInterestitialAd(null);
          loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createAppLovinInterstitialAd(int pos, Function? onAdLoadedCallback) {
  if (getAppLovinInterestitialAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  AppLovinInterstitialAd.load(
      adUnitId: _settings?.appLovin?.interestialId ?? "",
      adLoadCallback: AppLovinInterstitialAdLoadCallback(
        onAdLoaded: (AppLovinInterstitialAd ad) {
          print('$ad loaded');
          setAppLovinInterestitialAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('InterstitialAd failed to load: $error.');
          setAppLovinInterestitialAd(null);
          loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
        },
      ));
}

void _createRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getRewardedAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  print('RewardedAd _createRewardedAd Start.');

  RewardedAd.load(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      request: _request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('RewardedAd loaded.');
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdmanagerRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getRewardedAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }
  print('RewardedAd _createAdmanagerRewardedAd Start.');

  RewardedAd.loadWithAdManagerAdRequest(
      adUnitId: _settings?.googleAds?.rewardedId ?? "",
      adManagerRequest: _ad_manager_request,
      rewardedAdLoadCallback: RewardedAdLoadCallback(
        onAdLoaded: (RewardedAd ad) {
          print('RewardedAd loaded.');
          setRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAppLovinRewardedAd(
    int pos, Function? onAdLoadedCallback, Function? onAdFailedCallback) {
  if (getAppLovinRewardedAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  print('RewardedAd _createAppLovinRewardedAd Start.');

  AppLovinRewardedAd.load(
      adUnitId: _settings?.appLovin?.rewaredId ?? "",
      rewardedAdLoadCallback: AppLovinRewardedAdLoadCallback(
        onAdLoaded: (AppLovinRewardedAd ad) {
          print('RewardedAd loaded.');
          setAppLovinRewardedAd(ad);
          if (onAdLoadedCallback != null) {
            onAdLoadedCallback();
          }
        },
        onAdFailedToLoad: (LoadAdError error) {
          print('RewardedAd failed to load: $error');
          setAppLovinRewardedAd(null);
          loadRewardAds(
              pos: pos + 1,
              onAdLoadedCallback: onAdLoadedCallback,
              onAdFailedCallback: onAdFailedCallback);
        },
      ));
}

void _createAdManagerInterstitialAd(int pos, Function? onAdLoadedCallback) {
  if (getAdManagerInterstitialAd() != null) {
    if (onAdLoadedCallback != null) {
      onAdLoadedCallback();
    }
    return;
  }

  AdManagerInterstitialAd.load(
      adUnitId: _settings?.googleAds?.interestialId ?? "",
      request: _ad_manager_request,
      adLoadCallback: AdManagerInterstitialAdLoadCallback(
          onAdLoaded: (AdManagerInterstitialAd ad) {
        print('$ad loaded');
        setAdManagerInterstitialAd(ad);
        getAdManagerInterstitialAd()?.setImmersiveMode(true);
        if (onAdLoadedCallback != null) {
          onAdLoadedCallback();
        }
      }, onAdFailedToLoad: (LoadAdError error) {
        print('InterstitialAd failed to load: $error.');
        setAdManagerInterstitialAd(null);
        loadIntraAds(pos: pos + 1, onAdLoadedCallback: onAdLoadedCallback);
      }));
}

void _showInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getInterestitialAd() == null) {
    _showAdManagerInterstitialAd(callback);
    return;
  }

  getInterestitialAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (InterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (InterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent: (InterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getInterestitialAd()?.show();
  setInterestitialAd(null);
}

void _showAdManagerInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAdManagerInterstitialAd() == null) {
    _showAppLovinInterstitialAd(callback);
    return;
  }

  getAdManagerInterstitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AdManagerInterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AdManagerInterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      setInterestialAdTimeInterval(DateTime.now().millisecondsSinceEpoch);

      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AdManagerInterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAdManagerInterstitialAd()?.show();
  setAdManagerInterstitialAd(null);
}

void _showAppLovinInterstitialAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinInterestitialAd() == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  getAppLovinInterestitialAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinInterstitialAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AppLovinInterstitialAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
    onAdFailedToShowFullScreenContent:
        (AppLovinInterstitialAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      // _loadInterstitialAd(pos: pos + 1);
      if (callback != null) {
        callback();
      }
      loadIntraAds();
    },
  );
  getAppLovinInterestitialAd()?.show();
  setAppLovinInterestitialAd(null);
}

void _showRewardedAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getRewardedAd() == null) {
    _showAppLovinRewardedAd(callback);
    return;
  }

  getRewardedAd()?.fullScreenContentCallback = FullScreenContentCallback(
    onAdShowedFullScreenContent: (RewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (RewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (RewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (callback != null) {
        callback();
      }
      loadRewardAds();
    },
  );

  getRewardedAd()?.setImmersiveMode(true);
  setRewardShowCompleted(false);
  getRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
    setRewardShowCompleted(true);
  });
  setRewardedAd(null);
}

void _showAppLovinRewardedAd(Function? callback) {
  if (_settings == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  if (getAppLovinRewardedAd() == null) {
    if (callback != null) {
      callback();
    }
    return;
  }

  getAppLovinRewardedAd()?.fullScreenContentCallback =
      FullScreenContentCallback(
    onAdShowedFullScreenContent: (AppLovinRewardedAd ad) =>
        print('ad onAdShowedFullScreenContent.'),
    onAdDismissedFullScreenContent: (AppLovinRewardedAd ad) {
      print('$ad onAdDismissedFullScreenContent.');
      ad.dispose();
      loadRewardAds();
      if (IsRewardShowCompleted()) {
        if (callback != null) {
          callback();
        }
      }
    },
    onAdFailedToShowFullScreenContent: (AppLovinRewardedAd ad, AdError error) {
      print('$ad onAdFailedToShowFullScreenContent: $error');
      ad.dispose();
      if (callback != null) {
        callback();
      }
      loadRewardAds();
    },
  );

  setRewardShowCompleted(false);
  getAppLovinRewardedAd()?.show(
      onUserEarnedReward: (AdWithoutView ad, RewardItem reward) {
    print('$ad with reward $RewardItem(${reward.amount}, ${reward.type})');
    setRewardShowCompleted(true);
  });

  setAppLovinRewardedAd(null);
}
