import 'package:google_applovin_unity_ads/google_applovin_unity_ads.dart';

/// Listens for app foreground events and shows app open ads.
class AppLifecycleReactor {
  final AppOpenAdManager appOpenAdManager;
  final int appVersionCode;

  AppLifecycleReactor({required this.appOpenAdManager, required this.appVersionCode});

  void listenToAppStateChanges() {
    AppStateEventNotifier.startListening();
    AppStateEventNotifier.appStateStream
        .forEach((state) => _onAppStateChanged(state));
  }

  void _onAppStateChanged(AppState appState) {
    print('New AppState state: $appState');
    final _settings = getAdsSettings();
    if (_settings == null) {
      return;
    }

    if (_settings.adSetting?.appVersionCode == appVersionCode) {
      return;
    }

    if (!(_settings.adsSequence?.contains("google_ads") ?? false)  && !(_settings.adsSequence?.contains("ad_manager") ?? false)) {
      return;
    }

    if (appState == AppState.foreground) {
      if (_settings.googleAds?.isOpenAds ?? false) {
        appOpenAdManager.showAdIfAvailable();
      }
    }
  }
}